part of 'main_routes.dart';

class _Routes {
  final Build<HomeRoute> _build;
  final Watch? _watch;
  _Routes(this._build, {Watch? watch}) : _watch = watch;
  Routes call({List<RouteDefinition>? route, List<Listenable>? watch}) =>
      Routes(_build, [...?route], [...?_watch?.call(), ...?watch]);
  _RoutesAtAuthorsRoute atAuthorsRoute(Build<AuthorsRoute> build) =>
      _RoutesAtAuthorsRoute(build);
  _RoutesAtBooksRoute atBooksRoute(Build<BooksRoute> build) =>
      _RoutesAtBooksRoute(build);
  Object atSettingsRoute(Build build) => const Object();
}

class Routes {
  final AppRoutes<HomeRoute> _routes;
  late final AppRouterState _state;
  Routes(Build<HomeRoute> build, List route, List<Listenable> watch)
      : _routes = AppRoutes<HomeRoute>({
          '/authors': (_) => const AuthorsRoute(),
          '/books': (_) => const BooksRoute(),
          '/settings': (_) => const SettingsRoute(),
          '/authors/<>': (data) => AuthorRoute(
                id: data[0],
              ),
          '/books/<>': (data) => BookRoute(
                id: data[0],
              )
        }) {
    build(_routes);
    for (final def in route) {
      _routes.definitions.add(def);
    }
    _state = AppRouterState('Routes', _routes, watch);
  }
  late final routeParser = AppRouteParser(_routes);
  late final routerDelegate = AppRouterDelegate(_routes, _state, primary: true);
  AppRoute fromOrdinal(int ordinal) {
    switch (ordinal) {
      case 0:
        return AuthorsRoute();
      case 1:
        return BooksRoute();
      case 2:
        return SettingsRoute();
    }
    return ErrorRoute(
        message: 'invalid ordinal, $ordinal, requested from Routes');
  }

  void addFromOrdinal(int ordinal) => _state.add(fromOrdinal(ordinal));
  void putFromOrdinal(int ordinal) => _state.put(fromOrdinal(ordinal));
  void setFromOrdinal(int ordinal) => _state.set(fromOrdinal(ordinal));
  void setNewRoutePath(AppRoute route) => _state.setNewRoutePath(route);
  AppRoute get current => _state.currentLocal;
  void pop() => _state.pop();
  void addAuthors() => _state.add(const AuthorsRoute());
  void putAuthors() => _state.put(const AuthorsRoute());
  void setAuthors() => _state.set(const AuthorsRoute());
  void addBooks() => _state.add(const BooksRoute());
  void putBooks() => _state.put(const BooksRoute());
  void setBooks() => _state.set(const BooksRoute());
  void addSettings() => _state.add(const SettingsRoute());
  void putSettings() => _state.put(const SettingsRoute());
  void setSettings() => _state.set(const SettingsRoute());
}

class AuthorsRoute extends AppRoute {
  const AuthorsRoute();
  @override
  bool operator ==(Object? other) =>
      identical(this, other) || (other is AuthorsRoute);
  @override
  int get hashCode => runtimeType.hashCode;
  @override
  int toOrdinal() => 0;
  @override
  String toString() => '/authors';
}

class BooksRoute extends AppRoute {
  const BooksRoute();
  @override
  bool operator ==(Object? other) =>
      identical(this, other) || (other is BooksRoute);
  @override
  int get hashCode => runtimeType.hashCode;
  @override
  int toOrdinal() => 1;
  @override
  String toString() => '/books';
}

class SettingsRoute extends AppRoute {
  const SettingsRoute();
  @override
  bool operator ==(Object? other) =>
      identical(this, other) || (other is SettingsRoute);
  @override
  int get hashCode => runtimeType.hashCode;
  @override
  int toOrdinal() => 2;
  @override
  String toString() => '/settings';
}

class _RoutesAtAuthorsRoute {
  final Build<AuthorsRoute> _build;
  final List<Listenable> _watch;
  _RoutesAtAuthorsRoute(this._build, {List<Listenable>? watch})
      : _watch = watch ?? [];
  RoutesAtAuthorsRoute call(Routes parent) =>
      RoutesAtAuthorsRoute(parent._state, _build, _watch);
  Object atAuthorRoute(Build build) => const Object();
}

class RoutesAtAuthorsRoute {
  final AppRoutes<AuthorsRoute> _routes;
  late final AppRouterState _state;
  RoutesAtAuthorsRoute(
      AppRouterState parent, Build<AuthorsRoute> build, List<Listenable> watch)
      : _routes = AppRoutes<AuthorsRoute>() {
    build(_routes);
    _state =
        AppRouterState('RoutesAtAuthorsRoute', _routes, watch, parent: parent);
  }
  ChildRouter call() => ChildRouter(
      'RoutesAtAuthorsRoute', () => AppRouterDelegate(_routes, _state));
  AppRoute get current => _state.currentLocal;
  void pop() => _state.pop();
  void addAuthor({required String id}) => _state.add(AuthorRoute(id: id));
  void putAuthor({required String id}) => _state.put(AuthorRoute(id: id));
  void setAuthor({required String id}) => _state.set(AuthorRoute(id: id));
}

class AuthorRoute extends AppRoute {
  final String id;
  final authors = const AuthorsRoute();
  const AuthorRoute({required this.id});
  @override
  bool operator ==(Object? other) =>
      identical(this, other) || (other is AuthorRoute && id == other.id);
  @override
  int get hashCode => hashValues(runtimeType, id);
  @override
  int toOrdinal() => 0;
  @override
  List<AppRoute> toStack() => [...authors.toStack(), this];
  @override
  String toString() => '$authors/$id';
}

class _RoutesAtBooksRoute {
  final Build<BooksRoute> _build;
  final List<Listenable> _watch;
  _RoutesAtBooksRoute(this._build, {List<Listenable>? watch})
      : _watch = watch ?? [];
  RoutesAtBooksRoute call(Routes parent) =>
      RoutesAtBooksRoute(parent._state, _build, _watch);
  Object atBookRoute(Build build) => const Object();
}

class RoutesAtBooksRoute {
  final AppRoutes<BooksRoute> _routes;
  late final AppRouterState _state;
  RoutesAtBooksRoute(
      AppRouterState parent, Build<BooksRoute> build, List<Listenable> watch)
      : _routes = AppRoutes<BooksRoute>() {
    build(_routes);
    _state =
        AppRouterState('RoutesAtBooksRoute', _routes, watch, parent: parent);
  }
  ChildRouter call() => ChildRouter(
      'RoutesAtBooksRoute', () => AppRouterDelegate(_routes, _state));
  AppRoute get current => _state.currentLocal;
  void pop() => _state.pop();
  void addBook({required String id}) => _state.add(BookRoute(id: id));
  void putBook({required String id}) => _state.put(BookRoute(id: id));
  void setBook({required String id}) => _state.set(BookRoute(id: id));
}

class BookRoute extends AppRoute {
  final String id;
  final books = const BooksRoute();
  const BookRoute({required this.id});
  @override
  bool operator ==(Object? other) =>
      identical(this, other) || (other is BookRoute && id == other.id);
  @override
  int get hashCode => hashValues(runtimeType, id);
  @override
  int toOrdinal() => 0;
  @override
  List<AppRoute> toStack() => [...books.toStack(), this];
  @override
  String toString() => '$books/$id';
}
