import 'dart:async';

import 'package:flutter/foundation.dart';

abstract class StateListenable<T> extends Listenable {
  T? get state;
  const StateListenable();
}
class StreamListenable<T> implements StateListenable<T> {

  final Stream<T?> _stream;
  final T Function()? _orElse;
  final _listeners = <VoidCallback>[];

  StreamListenable({
    required Stream<T> stream,
    required T initialState,
    T Function()? orElse
  })
      : _stream = stream,
        _state = initialState,
        _orElse = orElse;

  StreamListenable.nullable({
    required Stream<T?> stream,
    T? initialState,
    T Function()? orElse
  })
      : _stream = stream,
        _state = initialState,
        _orElse = orElse;

  T? _state;
  @override
  T? get state => _state ?? _orElse?.call();

  StreamSubscription? _sub;

  @override
  void addListener(VoidCallback listener) {
    _sub ??= _stream.listen(_notify);
    _listeners.add(listener);
  }

  void cancel() {
    _sub?.cancel();
  }

  @override
  void removeListener(VoidCallback listener) {
    _listeners.remove(listener);
    if (_listeners.isEmpty) {
      cancel();
    }
  }

  void _notify(T? value) {
    _state = value;
    for (final listener in _listeners) {
      listener();
    }
  }
}
